"""Collection function resolver for synthetic sensors.

This module provides runtime resolution of collection functions that query
Home Assistant's entity registry to find entities matching specific patterns.

Supported collection patterns:
- regex: Pattern matching against entity IDs
- device_class: Filter by device class
- tags: Filter by entity tags/labels
- area: Filter by area assignment
- attribute: Filter by attribute conditions
- state: Filter by entity state values

Each pattern can be combined with aggregation functions like sum(), avg(), count(), etc.
All patterns support OR logic using pipe (|) syntax.
"""

from __future__ import annotations

import logging
import operator
import re

from homeassistant.core import HomeAssistant
from homeassistant.helpers import (
    area_registry as ar,
    device_registry as dr,
    entity_registry as er,
)

from .dependency_parser import DynamicQuery

_LOGGER = logging.getLogger(__name__)


class CollectionResolver:
    """Resolves collection function patterns to actual entity values."""

    def __init__(self, hass: HomeAssistant):
        """Initialize collection resolver with Home Assistant instance.

        Args:
            hass: Home Assistant instance for entity registry access
        """
        self._hass = hass

        # Try to get registries, but handle cases where they might not be available (testing)
        try:
            self._entity_registry: er.EntityRegistry | None = er.async_get(hass)
            self._device_registry: dr.DeviceRegistry | None = dr.async_get(hass)
            self._area_registry: ar.AreaRegistry | None = ar.async_get(hass)
        except (AttributeError, KeyError):
            # For testing or when registries aren't fully initialized
            self._entity_registry = None
            self._device_registry = None
            self._area_registry = None

        # Pattern for detecting entity references within collection patterns
        self._entity_reference_pattern = re.compile(r"\b(?:sensor|binary_sensor|input_number|input_boolean|input_select|input_text|switch|light|climate|cover|fan|lock|alarm_control_panel|vacuum|media_player|camera|weather|device_tracker|person|zone|automation|script|scene|group|timer|counter|sun)\.[a-zA-Z0-9_.]+\b")

    def resolve_collection(self, query: DynamicQuery) -> list[str]:
        """Resolve a dynamic query to a list of matching entity IDs.

        Args:
            query: Dynamic query specification

        Returns:
            List of entity IDs that match the query criteria
        """
        _LOGGER.debug("Resolving collection query: %s:%s", query.query_type, query.pattern)

        # First, resolve any entity references within the pattern
        resolved_pattern = self._resolve_entity_references_in_pattern(query.pattern)

        _LOGGER.debug("Pattern after entity resolution: %s", resolved_pattern)

        if query.query_type == "regex":
            return self._resolve_regex_pattern(resolved_pattern)
        elif query.query_type == "device_class":
            return self._resolve_device_class_pattern(resolved_pattern)
        elif query.query_type == "tags":
            return self._resolve_tags_pattern(resolved_pattern)
        elif query.query_type == "area":
            return self._resolve_area_pattern(resolved_pattern)
        elif query.query_type == "attribute":
            return self._resolve_attribute_pattern(resolved_pattern)
        elif query.query_type == "state":
            return self._resolve_state_pattern(resolved_pattern)
        else:
            _LOGGER.warning("Unknown collection query type: %s", query.query_type)
            return []

    def _resolve_entity_references_in_pattern(self, pattern: str) -> str:
        """Resolve entity references within a collection pattern.

        This method detects entity references like 'input_select.device_type' within
        collection patterns and replaces them with their current state values.

        Args:
            pattern: Collection pattern that may contain entity references

        Returns:
            Pattern with entity references resolved to their current values
        """

        def replace_entity_ref(match: re.Match[str]) -> str:
            """Replace an entity reference with its current state value."""
            entity_id = match.group(0)
            state = self._hass.states.get(entity_id)
            if state is not None:
                _LOGGER.debug("Resolving entity reference '%s' to value '%s'", entity_id, state.state)
                return str(state.state)
            else:
                _LOGGER.warning("Entity reference '%s' not found, keeping original", entity_id)
                return entity_id

        # Replace all entity references in the pattern
        resolved_pattern = self._entity_reference_pattern.sub(replace_entity_ref, pattern)

        return resolved_pattern

    def _resolve_regex_pattern(self, pattern: str) -> list[str]:
        """Resolve regex pattern against entity IDs.

        Args:
            pattern: Regular expression pattern to match (supports OR with pipe |)

        Returns:
            List of matching entity IDs
        """
        matching_entities: list[str] = []

        # Split by pipe (|) for OR logic
        regex_patterns = [p.strip() for p in pattern.split("|")]

        for regex_pattern in regex_patterns:
            try:
                regex = re.compile(regex_pattern)

                for entity_id in self._hass.states.entity_ids():
                    if regex.match(entity_id) and entity_id not in matching_entities:
                        matching_entities.append(entity_id)

            except re.error as e:
                _LOGGER.error("Invalid regex pattern '%s': %s", regex_pattern, e)
                continue

        _LOGGER.debug("Regex pattern '%s' matched %d entities", pattern, len(matching_entities))
        return matching_entities

    def _resolve_device_class_pattern(self, pattern: str) -> list[str]:
        """Resolve device_class pattern.

        Args:
            pattern: Device class to match (e.g., "temperature", "power", "door|window")

        Returns:
            List of matching entity IDs
        """
        matching_entities: list[str] = []

        # Split by pipe (|) for OR logic
        device_classes = [cls.strip() for cls in pattern.split("|")]

        for entity_id in self._hass.states.entity_ids():
            state = self._hass.states.get(entity_id)
            if state and hasattr(state, "attributes"):
                entity_device_class = state.attributes.get("device_class")
                if entity_device_class in device_classes:
                    matching_entities.append(entity_id)

        _LOGGER.debug("Device class pattern '%s' matched %d entities", pattern, len(matching_entities))
        return matching_entities

    def _resolve_tags_pattern(self, pattern: str) -> list[str]:
        """Resolve tags/labels pattern.

        Args:
            pattern: Tags to match using OR logic (e.g., "critical|important|warning")

        Returns:
            List of matching entity IDs
        """
        matching_entities: list[str] = []

        if not self._entity_registry:
            _LOGGER.warning("Entity registry not available for tags pattern resolution")
            return matching_entities

        # Split by pipe (|) for OR logic
        target_tags = [tag.strip() for tag in pattern.split("|")]

        for entity_entry in self._entity_registry.entities.values():
            entity_labels = set(entity_entry.labels) if entity_entry.labels else set()

            # Check if entity has any of the target tags (OR logic)
            if any(tag in entity_labels for tag in target_tags):
                matching_entities.append(entity_entry.entity_id)

        _LOGGER.debug("Tags pattern '%s' matched %d entities", pattern, len(matching_entities))
        return matching_entities

    def _resolve_area_pattern(self, pattern: str) -> list[str]:
        """Resolve area pattern with OR logic support.

        Args:
            pattern: Area names using OR logic (e.g., "living_room|kitchen|dining_room")

        Returns:
            List of matching entity IDs
        """
        matching_entities: list[str] = []

        if not self._area_registry or not self._entity_registry:
            _LOGGER.warning("Area or entity registry not available for area pattern resolution")
            return matching_entities

        # Split by pipe (|) for OR logic
        target_areas = [area_name.strip() for area_name in pattern.split("|")]

        # Find matching area IDs
        area_ids = set()
        for target_area_name in target_areas:
            # Handle device_class filter if present
            parts = target_area_name.split("device_class:")
            area_name = parts[0].strip()
            device_class_filter = parts[1].strip() if len(parts) > 1 else None

            # Find area by name
            for area in self._area_registry.areas.values():
                if area.name.lower() == area_name.lower():
                    area_ids.add((area.id, device_class_filter))
                    break

        # Find entities in any of the target areas
        for entity_entry in self._entity_registry.entities.values():
            entity_area_id = entity_entry.area_id

            # Check device area if entity doesn't have direct area assignment
            if not entity_area_id and entity_entry.device_id and self._device_registry:
                device_entry = self._device_registry.devices.get(entity_entry.device_id)
                if device_entry:
                    entity_area_id = device_entry.area_id

            # Check if entity is in any of the target areas
            for area_id, device_class_filter in area_ids:
                if entity_area_id == area_id:
                    # Apply device class filter if specified
                    if device_class_filter:
                        state = self._hass.states.get(entity_entry.entity_id)
                        if state and hasattr(state, "attributes"):
                            entity_device_class = state.attributes.get("device_class")
                            if entity_device_class == device_class_filter:
                                matching_entities.append(entity_entry.entity_id)
                    else:
                        matching_entities.append(entity_entry.entity_id)
                    break  # Avoid adding the same entity multiple times

        _LOGGER.debug("Area pattern '%s' matched %d entities", pattern, len(matching_entities))
        return matching_entities

    def _resolve_attribute_pattern(self, pattern: str) -> list[str]:
        """Resolve attribute condition pattern with OR logic support.

        Args:
            pattern: Attribute conditions using OR logic (e.g., "battery_level<20|online=false")

        Returns:
            List of matching entity IDs
        """
        matching_entities: list[str] = []
        attribute_conditions = [condition.strip() for condition in pattern.split("|")]

        for condition in attribute_conditions:
            condition_matches = self._resolve_single_attribute_condition(condition, matching_entities)
            matching_entities.extend(condition_matches)

        _LOGGER.debug("Attribute pattern '%s' matched %d entities", pattern, len(matching_entities))
        return matching_entities

    def _resolve_single_attribute_condition(self, condition: str, existing_matches: list[str]) -> list[str]:
        """Resolve a single attribute condition.

        Args:
            condition: Single attribute condition (e.g., "battery_level<20")
            existing_matches: Entities already matched to avoid duplicates

        Returns:
            List of newly matching entity IDs
        """
        matches: list[str] = []

        # Parse the condition
        parsed = self._parse_attribute_condition(condition)
        if not parsed:
            _LOGGER.warning("Invalid attribute condition '%s'", condition)
            return matches

        attribute_name, op, expected_value = parsed

        # Check entities for matching attribute
        for entity_id in self._hass.states.entity_ids():
            if entity_id in existing_matches:
                continue  # Already matched by previous condition

            if self._entity_matches_attribute_condition(entity_id, attribute_name, op, expected_value):
                matches.append(entity_id)

        return matches

    def _parse_attribute_condition(self, condition: str) -> tuple[str, str, bool | float | int | str] | None:
        """Parse a single attribute condition into components.

        Args:
            condition: Attribute condition string

        Returns:
            Tuple of (attribute_name, operator, expected_value) or None if invalid
        """
        for op in ["<=", ">=", "!=", "<", ">", "="]:
            if op in condition:
                attribute_name, value_str = condition.split(op, 1)
                attribute_name = attribute_name.strip()
                value_str = value_str.strip()
                expected_value = self._convert_value_string(value_str)
                return attribute_name, op, expected_value
        return None

    def _entity_matches_attribute_condition(self, entity_id: str, attribute_name: str, op: str, expected_value: bool | float | int | str) -> bool:
        """Check if an entity matches an attribute condition.

        Args:
            entity_id: Entity to check
            attribute_name: Name of attribute to check
            op: Comparison operator
            expected_value: Expected value to compare against

        Returns:
            True if entity matches condition
        """
        state = self._hass.states.get(entity_id)
        if not state or not hasattr(state, "attributes"):
            return False

        actual_value = state.attributes.get(attribute_name)
        if actual_value is None:
            return False

        # Convert actual value to same type as expected
        try:
            if isinstance(expected_value, bool):
                converted_value: bool | float | str = str(actual_value).lower() == "true"
            elif isinstance(expected_value, (int, float)):
                converted_value = float(actual_value)
            else:
                converted_value = str(actual_value)
        except (ValueError, TypeError):
            return False

        return self._compare_values(converted_value, op, expected_value)

    def _convert_value_string(self, value_str: str) -> bool | float | int | str:
        """Convert a string value to appropriate type.

        Args:
            value_str: String to convert

        Returns:
            Converted value (bool, float, int, or str)
        """
        try:
            if value_str.lower() in ("true", "false"):
                return value_str.lower() == "true"
            elif "." in value_str:
                return float(value_str)
            else:
                return int(value_str)
        except ValueError:
            return value_str  # Keep as string

    def _resolve_state_pattern(self, pattern: str) -> list[str]:
        """Resolve state condition pattern with OR logic support.

        Args:
            pattern: State conditions using OR logic (e.g., ">100|=on|<50")

        Returns:
            List of matching entity IDs
        """
        matching_entities: list[str] = []
        state_conditions = [condition.strip() for condition in pattern.split("|")]

        for condition in state_conditions:
            condition_matches = self._resolve_single_state_condition(condition, matching_entities)
            matching_entities.extend(condition_matches)

        _LOGGER.debug("State pattern '%s' matched %d entities", pattern, len(matching_entities))
        return matching_entities

    def _resolve_single_state_condition(self, condition: str, existing_matches: list[str]) -> list[str]:
        """Resolve a single state condition.

        Args:
            condition: Single state condition (e.g., ">100")
            existing_matches: Entities already matched to avoid duplicates

        Returns:
            List of newly matching entity IDs
        """
        matches: list[str] = []

        # Parse the condition
        parsed = self._parse_state_condition(condition)
        if not parsed:
            _LOGGER.warning("Invalid state condition '%s'", condition)
            return matches

        op, expected_value = parsed

        # Check entities for matching state
        for entity_id in self._hass.states.entity_ids():
            if entity_id in existing_matches:
                continue  # Already matched by previous condition

            if self._entity_matches_state_condition(entity_id, op, expected_value):
                matches.append(entity_id)

        return matches

    def _parse_state_condition(self, condition: str) -> tuple[str, bool | float | int | str] | None:
        """Parse a single state condition into components.

        Args:
            condition: State condition string

        Returns:
            Tuple of (operator, expected_value) or None if invalid
        """
        for op in ["<=", ">=", "!=", "<", ">", "="]:
            if op in condition:
                value_str = condition.replace(op, "", 1).strip()
                expected_value = self._convert_value_string(value_str)
                return op, expected_value
        return None

    def _entity_matches_state_condition(self, entity_id: str, op: str, expected_value: bool | float | int | str) -> bool:
        """Check if an entity matches a state condition.

        Args:
            entity_id: Entity to check
            op: Comparison operator
            expected_value: Expected value to compare against

        Returns:
            True if entity matches condition
        """
        state = self._hass.states.get(entity_id)
        if not state or state.state in ("unknown", "unavailable", "none"):
            return False

        # Convert actual state to same type as expected
        try:
            if isinstance(expected_value, bool):
                converted_state: bool | float | str = str(state.state).lower() == "true"
            elif isinstance(expected_value, (int, float)):
                converted_state = float(state.state)
            else:
                converted_state = str(state.state)
        except (ValueError, TypeError):
            return False

        return self._compare_values(converted_state, op, expected_value)

    def _compare_values(self, actual: bool | float | str, op: str, expected: bool | float | int | str) -> bool:
        """Compare two values using the specified operator.

        Args:
            actual: Actual value from entity
            op: Comparison operator (=, !=, <, >, <=, >=)
            expected: Expected value from pattern

        Returns:
            True if comparison matches, False otherwise
        """
        # Use operator module for type-safe comparisons
        operations = {
            "=": operator.eq,
            "!=": operator.ne,
            "<": operator.lt,
            ">": operator.gt,
            "<=": operator.le,
            ">=": operator.ge,
        }

        try:
            if op in operations:
                result = operations[op](actual, expected)
                return bool(result)
            return False
        except TypeError:
            # Type mismatch, try string comparison
            return str(actual) == str(expected) if op == "=" else False

    def get_entity_values(self, entity_ids: list[str]) -> list[float]:
        """Get numeric values for a list of entity IDs.

        Args:
            entity_ids: List of entity IDs to get values for

        Returns:
            List of numeric values, excluding unavailable/non-numeric states
        """
        values = []

        for entity_id in entity_ids:
            state = self._hass.states.get(entity_id)
            if state and state.state not in ("unknown", "unavailable", "none"):
                try:
                    value = float(state.state)
                    values.append(value)
                except (ValueError, TypeError):
                    _LOGGER.debug("Skipping non-numeric state for %s: %s", entity_id, state.state)

        _LOGGER.debug("Retrieved %d numeric values from %d entities", len(values), len(entity_ids))
        return values
